@@ -1,4 +1,4 @@
 Sys.setenv("R_TESTS" = "")
-if(require(testthat) & require(fpp) & require(forecast)){
+if(require(testthat) & require(forecast)){
   test_check("forecastHybrid")
   }
